@@ -1,4 +1,5 @@
 import socket
+from concurrent.futures import CancelledError
 
 import pytest
 from botocore.awsrequest import (
@@ -12,6 +13,7 @@ from botocore.exceptions import (
     ProxyConnectionError,
 )
 from botocore.httpsession import (
+    BUFFER_SIZE,
     ProxyConfiguration,
     URLLib3Session,
     get_cert_path,
@@ -148,7 +150,6 @@ class TestURLLib3Session(unittest.TestCase):
 
     def _assert_manager_call(self, manager, *assert_args, **assert_kwargs):
         call_kwargs = {
-            'strict': True,
             'maxsize': mock.ANY,
             'timeout': mock.ANY,
             'ssl_context': mock.ANY,
@@ -156,6 +157,8 @@ class TestURLLib3Session(unittest.TestCase):
             'cert_file': None,
             'key_file': None,
         }
+        if BUFFER_SIZE:
+            call_kwargs['blocksize'] = BUFFER_SIZE
         call_kwargs.update(assert_kwargs)
         manager.assert_called_with(*assert_args, **call_kwargs)
 
@@ -463,6 +466,12 @@ class TestURLLib3Session(unittest.TestCase):
         with pytest.raises(ConnectionClosedError):
             self.make_request_with_error(error)
 
+    def test_catches_cancelled_error(self):
+        self.connection.urlopen.side_effect = CancelledError()
+        session = URLLib3Session()
+        with pytest.raises(CancelledError):
+            session.send(self.request.prepare())
+
     def test_catches_proxy_error(self):
         self.connection.urlopen.side_effect = ProxyError('test', None)
         session = URLLib3Session(
