@@ -989,7 +989,7 @@ class TestArtifactExporter(BaseYAMLTest):
         "awscli.customizations.cloudformation.artifact_exporter.yaml_parse"
     )
     def test_template_export_metadata(self, yaml_parse_mock):
-        parent_dir = os.path.sep
+        parent_dir = os.path.abspath(os.path.sep)
         template_dir = os.path.join(parent_dir, 'foo', 'bar')
         template_path = os.path.join(template_dir, 'path')
         template_str = self.example_yaml_template()
@@ -1051,7 +1051,7 @@ class TestArtifactExporter(BaseYAMLTest):
         "awscli.customizations.cloudformation.artifact_exporter.yaml_parse"
     )
     def test_template_export(self, yaml_parse_mock):
-        parent_dir = os.path.sep
+        parent_dir = os.path.abspath(os.path.sep)
         template_dir = os.path.join(parent_dir, 'foo', 'bar')
         template_path = os.path.join(template_dir, 'path')
         template_str = self.example_yaml_template()
@@ -1122,7 +1122,7 @@ class TestArtifactExporter(BaseYAMLTest):
         "awscli.customizations.cloudformation.artifact_exporter.yaml_parse"
     )
     def test_template_export_foreach_valid(self, yaml_parse_mock):
-        parent_dir = os.path.sep
+        parent_dir = os.path.abspath(os.path.sep)
         template_dir = os.path.join(parent_dir, 'foo', 'bar')
         template_path = os.path.join(template_dir, 'path')
         template_str = self.example_yaml_template()
@@ -1227,7 +1227,7 @@ class TestArtifactExporter(BaseYAMLTest):
         "awscli.customizations.cloudformation.artifact_exporter.yaml_parse"
     )
     def test_template_export_foreach_invalid(self, yaml_parse_mock):
-        parent_dir = os.path.sep
+        parent_dir = os.path.abspath(os.path.sep)
         template_dir = os.path.join(parent_dir, 'foo', 'bar')
         template_path = os.path.join(template_dir, 'path')
         template_str = self.example_yaml_template()
@@ -1289,7 +1289,7 @@ class TestArtifactExporter(BaseYAMLTest):
         "awscli.customizations.cloudformation.artifact_exporter.yaml_parse"
     )
     def test_template_global_export(self, yaml_parse_mock):
-        parent_dir = os.path.sep
+        parent_dir = os.path.abspath(os.path.sep)
         template_dir = os.path.join(parent_dir, 'foo', 'bar')
         template_path = os.path.join(template_dir, 'path')
         template_str = self.example_yaml_template()
